@@ -1,6 +1,6 @@
 'use strict';
 
-let Long = require('./long');
+var Long = require('./long');
 
 var PARSE_STRING_REGEXP = /^(\+|-)?(\d+|(\d*\.\d*))?(E|e)?([-+])?(\d+)?$/;
 var PARSE_INF_REGEXP = /^(\+|-)?(Infinity|inf)$/i;
@@ -79,12 +79,13 @@ var isDigit = function(value) {
 var divideu128 = function(value) {
   var DIVISOR = Long.fromNumber(1000 * 1000 * 1000);
   var _rem = Long.fromNumber(0);
+  var i = 0;
 
   if (!value.parts[0] && !value.parts[1] && !value.parts[2] && !value.parts[3]) {
     return { quotient: value, rem: _rem };
   }
 
-  for (let i = 0; i <= 3; i++) {
+  for (i = 0; i <= 3; i++) {
     // Adjust remainder to match value of next dividend
     _rem = _rem.shiftLeft(32);
     // Add the divided to _rem
@@ -141,9 +142,34 @@ var lessThan = function(left, right) {
   return false;
 };
 
-var invalidErr = function(string, message) {
-  throw new TypeError('"${string}" not a valid Decimal128 string - ' + message);
-};
+// var longtoHex = function(value) {
+//   var buffer = new Buffer(8);
+//   var index = 0;
+//   // Encode the low 64 bits of the decimal
+//   // Encode low bits
+//   buffer[index++] = value.low_ & 0xff;
+//   buffer[index++] = (value.low_ >> 8) & 0xff;
+//   buffer[index++] = (value.low_ >> 16) & 0xff;
+//   buffer[index++] = (value.low_ >> 24) & 0xff;
+//   // Encode high bits
+//   buffer[index++] = value.high_ & 0xff;
+//   buffer[index++] = (value.high_ >> 8) & 0xff;
+//   buffer[index++] = (value.high_ >> 16) & 0xff;
+//   buffer[index++] = (value.high_ >> 24) & 0xff;
+//   return buffer.reverse().toString('hex');
+// };
+
+// var int32toHex = function(value) {
+//   var buffer = new Buffer(4);
+//   var index = 0;
+//   // Encode the low 64 bits of the decimal
+//   // Encode low bits
+//   buffer[index++] = value & 0xff;
+//   buffer[index++] = (value >> 8) & 0xff;
+//   buffer[index++] = (value >> 16) & 0xff;
+//   buffer[index++] = (value >> 24) & 0xff;
+//   return buffer.reverse().toString('hex');
+// };
 
 /**
  * A class representation of the BSON Decimal128 type.
@@ -152,10 +178,10 @@ var invalidErr = function(string, message) {
  * @param {Buffer} bytes a buffer containing the raw Decimal128 bytes.
  * @return {Double}
  */
-function Decimal128(bytes) {
+var Decimal128 = function(bytes) {
   this._bsontype = 'Decimal128';
   this.bytes = bytes;
-}
+};
 
 /**
  * Create a Decimal128 instance from a string representation
@@ -206,11 +232,14 @@ Decimal128.fromString = function(string) {
   // Read index
   var index = 0;
 
+  // Trim the string
+  string = string.trim();
+
   // Naively prevent against REDOS attacks.
   // TODO: implementing a custom parsing for this, or refactoring the regex would yield
   //       further gains.
   if (string.length >= 7000) {
-    throw new TypeError('' + string + ' not a valid Decimal128 string');
+    throw new Error('' + string + ' not a valid Decimal128 string');
   }
 
   // Results
@@ -220,30 +249,12 @@ Decimal128.fromString = function(string) {
 
   // Validate the string
   if ((!stringMatch && !infMatch && !nanMatch) || string.length === 0) {
-    throw new TypeError('' + string + ' not a valid Decimal128 string');
+    throw new Error('' + string + ' not a valid Decimal128 string');
   }
 
-  if (stringMatch) {
-    // full_match = stringMatch[0]
-    // sign = stringMatch[1]
-
-    var unsignedNumber = stringMatch[2];
-    // stringMatch[3] is undefined if a whole number (ex "1", 12")
-    // but defined if a number w/ decimal in it (ex "1.0, 12.2")
-
-    var e = stringMatch[4];
-    var expSign = stringMatch[5];
-    var expNumber = stringMatch[6];
-
-    // they provided e, but didn't give an exponent number. for ex "1e"
-    if (e && expNumber === undefined) invalidErr(string, 'missing exponent power');
-
-    // they provided e, but didn't give a number before it. for ex "e1"
-    if (e && unsignedNumber === undefined) invalidErr(string, 'missing exponent base');
-
-    if (e === undefined && (expSign || expNumber)) {
-      invalidErr(string, 'missing e before exponent');
-    }
+  // Check if we have an illegal exponent format
+  if (stringMatch && stringMatch[4] && stringMatch[2] === undefined) {
+    throw new Error('' + string + ' not a valid Decimal128 string');
   }
 
   // Get the negative or positive sign
@@ -263,7 +274,9 @@ Decimal128.fromString = function(string) {
   // Read all the digits
   while (isDigit(string[index]) || string[index] === '.') {
     if (string[index] === '.') {
-      if (sawRadix) invalidErr(string, 'contains multiple periods');
+      if (sawRadix) {
+        return new Decimal128(new Buffer(NAN_BUFFER));
+      }
 
       sawRadix = true;
       index = index + 1;
@@ -284,14 +297,21 @@ Decimal128.fromString = function(string) {
       }
     }
 
-    if (foundNonZero) nDigits = nDigits + 1;
-    if (sawRadix) radixPosition = radixPosition + 1;
+    if (foundNonZero) {
+      nDigits = nDigits + 1;
+    }
+
+    if (sawRadix) {
+      radixPosition = radixPosition + 1;
+    }
 
     nDigitsRead = nDigitsRead + 1;
     index = index + 1;
   }
 
-  if (sawRadix && !nDigitsRead) throw new TypeError('' + string + ' not a valid Decimal128 string');
+  if (sawRadix && !nDigitsRead) {
+    throw new Error('' + string + ' not a valid Decimal128 string');
+  }
 
   // Read exponent if exists
   if (string[index] === 'e' || string[index] === 'E') {
@@ -299,7 +319,9 @@ Decimal128.fromString = function(string) {
     var match = string.substr(++index).match(EXPONENT_REGEX);
 
     // No digits read
-    if (!match || !match[2]) return new Decimal128(new Buffer(NAN_BUFFER));
+    if (!match || !match[2]) {
+      return new Decimal128(new Buffer(NAN_BUFFER));
+    }
 
     // Get exponent
     exponent = parseInt(match[0], 10);
@@ -309,7 +331,9 @@ Decimal128.fromString = function(string) {
   }
 
   // Return not a number
-  if (string[index]) return new Decimal128(new Buffer(NAN_BUFFER));
+  if (string[index]) {
+    return new Decimal128(new Buffer(NAN_BUFFER));
+  }
 
   // Done reading input
   // Find first non-zero digit in digits
@@ -325,7 +349,8 @@ Decimal128.fromString = function(string) {
   } else {
     lastDigit = nDigitsStored - 1;
     significantDigits = nDigits;
-    if (significantDigits !== 1) {
+
+    if (exponent !== 0 && significantDigits !== 1) {
       while (string[firstNonZero + significantDigits - 1] === '0') {
         significantDigits = significantDigits - 1;
       }
@@ -354,15 +379,17 @@ Decimal128.fromString = function(string) {
       if (digitsString.match(/^0+$/)) {
         exponent = EXPONENT_MAX;
         break;
+      } else {
+        return new Decimal128(new Buffer(isNegative ? INF_NEGATIVE_BUFFER : INF_POSITIVE_BUFFER));
       }
-      invalidErr(string, 'overflow');
     }
+
     exponent = exponent - 1;
   }
 
   while (exponent < EXPONENT_MIN || nDigitsStored < nDigits) {
-    // Shift last digit. can only do this if < significant digits than # stored.
-    if (lastDigit === 0 && significantDigits < nDigitsStored) {
+    // Shift last digit
+    if (lastDigit === 0) {
       exponent = EXPONENT_MIN;
       significantDigits = 0;
       break;
@@ -384,25 +411,21 @@ Decimal128.fromString = function(string) {
       if (digitsString.match(/^0+$/)) {
         exponent = EXPONENT_MAX;
         break;
+      } else {
+        return new Decimal128(new Buffer(isNegative ? INF_NEGATIVE_BUFFER : INF_POSITIVE_BUFFER));
       }
-      invalidErr(string, 'overflow');
     }
   }
 
   // Round
   // We've normalized the exponent, but might still need to round.
-  if (lastDigit - firstDigit + 1 < significantDigits) {
+  if (lastDigit - firstDigit + 1 < significantDigits && string[significantDigits] !== '0') {
     var endOfString = nDigitsRead;
 
     // If we have seen a radix point, 'string' is 1 longer than we have
     // documented with ndigits_read, so inc the position of the first nonzero
     // digit and the position that digits are read to.
-    if (sawRadix) {
-      firstNonZero = firstNonZero + 1;
-      endOfString = endOfString + 1;
-    }
-    // if negative, we need to increment again to account for - sign at start.
-    if (isNegative) {
+    if (sawRadix && exponent === EXPONENT_MIN) {
       firstNonZero = firstNonZero + 1;
       endOfString = endOfString + 1;
     }
@@ -412,8 +435,10 @@ Decimal128.fromString = function(string) {
 
     if (roundDigit >= 5) {
       roundBit = 1;
+
       if (roundDigit === 5) {
         roundBit = digits[lastDigit] % 2 === 1;
+
         for (i = firstNonZero + lastDigit + 2; i < endOfString; i++) {
           if (parseInt(string[i], 10)) {
             roundBit = 1;
@@ -441,6 +466,8 @@ Decimal128.fromString = function(string) {
               );
             }
           }
+        } else {
+          break;
         }
       }
     }
@@ -607,6 +634,7 @@ Decimal128.prototype.toString = function() {
   // temporary storage for significand decoding
   var significand128 = { parts: new Array(4) };
   // indexing variables
+  i;
   var j, k;
 
   // Output string
@@ -728,19 +756,9 @@ Decimal128.prototype.toString = function() {
   // has trailing zeros.  However, we *cannot* output these trailing zeros,
   // because doing so would change the precision of the value, and would
   // change stored data if the string converted number is round tripped.
+
   if (scientific_exponent >= 34 || scientific_exponent <= -7 || exponent > 0) {
     // Scientific format
-
-    // if there are too many significant digits, we should just be treating numbers
-    // as + or - 0 and using the non-scientific exponent (this is for the "invalid
-    // representation should be treated as 0/-0" spec cases in decimal128-1.json)
-    if (significand_digits > 34) {
-      string.push(0);
-      if (exponent > 0) string.push('E+' + exponent);
-      else if (exponent < 0) string.push('E' + exponent);
-      return string.join('');
-    }
-
     string.push(significand[index++]);
     significand_digits = significand_digits - 1;
 
