@@ -1,4 +1,3 @@
-'use strict';
 /**
  * A class representation of a BSON Int32 type.
  *
@@ -6,12 +5,12 @@
  * @param {number} value the number we want to represent as an int32.
  * @return {Int32}
  */
-function Int32(value) {
+var Int32 = function(value) {
   if (!(this instanceof Int32)) return new Int32(value);
 
   this._bsontype = 'Int32';
   this.value = value;
-}
+};
 
 /**
  * Access the number value.
