@@ -2,11 +2,11 @@
 
 var writeIEEE754 = require('../float_parser').writeIEEE754,
   Long = require('../long').Long,
-  Map = require('../map'),
   MinKey = require('../min_key').MinKey,
+  Map = require('../map'),
   Binary = require('../binary').Binary;
 
-const normalizedFunctionString = require('./utils').normalizedFunctionString;
+var normalizedFunctionString = require('./utils').normalizedFunctionString;
 
 // try {
 //   var _Buffer = Uint8Array;
@@ -15,6 +15,7 @@ const normalizedFunctionString = require('./utils').normalizedFunctionString;
 // }
 
 var regexp = /\x00/; // eslint-disable-line no-control-regex
+var ignoreKeys = ['$db', '$ref', '$id', '$clusterTime'];
 
 // To ensure that 0.4 of node works correctly
 var isDate = function isDate(d) {
@@ -129,12 +130,10 @@ var serializeNumber = function(buffer, key, value, index, isArray) {
 var serializeNull = function(buffer, key, value, index, isArray) {
   // Set long type
   buffer[index++] = BSON.BSON_DATA_NULL;
-
   // Number of written bytes
   var numberOfWrittenBytes = !isArray
     ? buffer.write(key, index, 'utf8')
     : buffer.write(key, index, 'ascii');
-
   // Encode the name
   index = index + numberOfWrittenBytes;
   buffer[index++] = 0;
@@ -191,7 +190,6 @@ var serializeRegExp = function(buffer, key, value, index, isArray) {
   var numberOfWrittenBytes = !isArray
     ? buffer.write(key, index, 'utf8')
     : buffer.write(key, index, 'ascii');
-
   // Encode the name
   index = index + numberOfWrittenBytes;
   buffer[index++] = 0;
@@ -203,10 +201,9 @@ var serializeRegExp = function(buffer, key, value, index, isArray) {
   // Write zero
   buffer[index++] = 0x00;
   // Write the parameters
-  if (value.ignoreCase) buffer[index++] = 0x69; // i
   if (value.global) buffer[index++] = 0x73; // s
+  if (value.ignoreCase) buffer[index++] = 0x69; // i
   if (value.multiline) buffer[index++] = 0x6d; // m
-
   // Add ending zero
   buffer[index++] = 0x00;
   return index;
@@ -288,7 +285,7 @@ var serializeObjectId = function(buffer, key, value, index, isArray) {
   } else if (value.id && value.id.copy) {
     value.id.copy(buffer, index, 0, 12);
   } else {
-    throw new TypeError('object [' + JSON.stringify(value) + '] is not a valid ObjectId');
+    throw new Error('object [' + JSON.stringify(value) + '] is not a valid ObjectId');
   }
 
   // Ajust index
@@ -360,6 +357,7 @@ var serializeObject = function(
   );
   // Pop stack
   path.pop();
+  // Write size
   return endIndex;
 };
 
@@ -424,20 +422,16 @@ var serializeInt32 = function(buffer, key, value, index, isArray) {
 var serializeDouble = function(buffer, key, value, index, isArray) {
   // Encode as double
   buffer[index++] = BSON.BSON_DATA_NUMBER;
-
   // Number of written bytes
   var numberOfWrittenBytes = !isArray
     ? buffer.write(key, index, 'utf8')
     : buffer.write(key, index, 'ascii');
-
   // Encode the name
   index = index + numberOfWrittenBytes;
   buffer[index++] = 0;
-
   // Write float
-  writeIEEE754(buffer, value.value, index, 'little', 52, 8);
-
-  // Adjust index
+  writeIEEE754(buffer, value, index, 'little', 52, 8);
+  // Ajust index
   index = index + 8;
   return index;
 };
@@ -638,15 +632,34 @@ var serializeDBRef = function(buffer, key, value, index, depth, serializeFunctio
 
   var startIndex = index;
   var endIndex;
-  var output = {
-    $ref: value.collection,
-    $id: value.oid
-  };
 
-  if (value.db != null) output.$db = value.db;
-
-  output = Object.assign(output, value.fields);
-  endIndex = serializeInto(buffer, output, false, index, depth + 1, serializeFunctions);
+  // Serialize object
+  if (null != value.db) {
+    endIndex = serializeInto(
+      buffer,
+      {
+        $ref: value.namespace,
+        $id: value.oid,
+        $db: value.db
+      },
+      false,
+      index,
+      depth + 1,
+      serializeFunctions
+    );
+  } else {
+    endIndex = serializeInto(
+      buffer,
+      {
+        $ref: value.namespace,
+        $id: value.oid
+      },
+      false,
+      index,
+      depth + 1,
+      serializeFunctions
+    );
+  }
 
   // Calculate object size
   var size = endIndex - startIndex;
@@ -677,6 +690,7 @@ var serializeInto = function serializeInto(
 
   // Start place to serialize into
   var index = startingIndex + 4;
+  // var self = this;
 
   // Special case isArray
   if (Array.isArray(object)) {
@@ -687,7 +701,7 @@ var serializeInto = function serializeInto(
 
       // Is there an override value
       if (value && value.toBSON) {
-        if (typeof value.toBSON !== 'function') throw new TypeError('toBSON is not a function');
+        if (typeof value.toBSON !== 'function') throw new Error('toBSON is not a function');
         value = value.toBSON();
       }
 
@@ -785,7 +799,7 @@ var serializeInto = function serializeInto(
       type = typeof value;
 
       // Check the key and throw error if it's illegal
-      if (key !== '$db' && key !== '$ref' && key !== '$id') {
+      if (typeof key === 'string' && ignoreKeys.indexOf(key) === -1) {
         if (key.match(regexp) != null) {
           // The BSON spec doesn't allow keys with null bytes because keys are
           // null-terminated.
@@ -809,6 +823,7 @@ var serializeInto = function serializeInto(
         index = serializeBoolean(buffer, key, value, index);
       } else if (value instanceof Date || isDate(value)) {
         index = serializeDate(buffer, key, value, index);
+        // } else if (value === undefined && ignoreUndefined === true) {
       } else if (value === null || (value === undefined && ignoreUndefined === false)) {
         index = serializeNull(buffer, key, value, index);
       } else if (value['_bsontype'] === 'ObjectID') {
@@ -866,10 +881,10 @@ var serializeInto = function serializeInto(
   } else {
     // Did we provide a custom serialization method
     if (object.toBSON) {
-      if (typeof object.toBSON !== 'function') throw new TypeError('toBSON is not a function');
+      if (typeof object.toBSON !== 'function') throw new Error('toBSON is not a function');
       object = object.toBSON();
       if (object != null && typeof object !== 'object')
-        throw new TypeError('toBSON function did not return an object');
+        throw new Error('toBSON function did not return an object');
     }
 
     // Iterate over all the keys
@@ -877,7 +892,7 @@ var serializeInto = function serializeInto(
       value = object[key];
       // Is there an override value
       if (value && value.toBSON) {
-        if (typeof value.toBSON !== 'function') throw new TypeError('toBSON is not a function');
+        if (typeof value.toBSON !== 'function') throw new Error('toBSON is not a function');
         value = value.toBSON();
       }
 
@@ -885,7 +900,7 @@ var serializeInto = function serializeInto(
       type = typeof value;
 
       // Check the key and throw error if it's illegal
-      if (key !== '$db' && key !== '$ref' && key !== '$id') {
+      if (typeof key === 'string' && ignoreKeys.indexOf(key) === -1) {
         if (key.match(regexp) != null) {
           // The BSON spec doesn't allow keys with null bytes because keys are
           // null-terminated.
