@@ -5,9 +5,10 @@
  * @param {Function} fn The function to stringify
  */
 function normalizedFunctionString(fn) {
-  return fn.toString().replace('function(', 'function (');
+  return fn.toString().replace(/function *\(/, 'function (');
 }
 
 module.exports = {
-  normalizedFunctionString
+  normalizedFunctionString: normalizedFunctionString
 };
+
